%% Day 3 Homework solution for problems 5 - 19

clear
close all

% Load data and assign to a struct named S. S will have two fields: data and
% textdata. We can access them with the dot operator, so S.data or
% S.textdata.
S = importdata('AneuploidStrains_RNAseq+gDNA_NormToEuploid.xlsx');

%Extract headers
headers = S.textdata(1,:);

%The first column are the gene IDs. The second column has more info about
%each gene. Columns 3 to 20 are for the numeric data

%% Find DNA and RNA columns

% Matlab 2016 added a function named 'contains' that will find strings in
% a cell array of strings. If your version is older, then use the 
% commented out lines below instead.

DNAind = contains(headers,'YPS1009') & contains(headers,'DNA');
RNAind = contains(headers,'YPS1009') & contains(headers,'RNA');

% Use the lines below if 'contains' doesn't work. Highlight all the lines 
% and press Cmd+T or Ctrl+T to remove the '%' symbols. Pressing Cmd+R or
% Ctrl+R will add '%' to each line

% % Use cellfun to apply the isempty function on each cell returned by
% % strfind. The ~ is used since we want the non-empty results, i.e. when
% % 'DNA' or 'RNA' are found
% DNAind = ~cellfun(@isempty,strfind(headers,'YPS1009')) & ...
%        ~cellfun(@isempty,strfind(headers,'DNA'));
% RNAind = ~cellfun(@isempty,strfind(headers,'YPS1009')) & ...
%        ~cellfun(@isempty,strfind(headers,'RNA'));   

% Find the number of text columns by finding the first empty column (which
% is the start of the numeric data. Then subtract 1 to get the number
nTextColumns = find(strcmp(S.textdata(3,:),''),1) - 1;

% Determine which columns these are in the numeric data by subtracting
% the number of text columns 
DNAcolumn = find(DNAind) - nTextColumns
RNAcolumns = find(RNAind) - nTextColumns

%% Find the right letter for chromosome 12

% This is one way of getting the letters A to P. Characters in Matlab are
% really just numbers, so you can use the colon operator on them. You could
% also just type out the letters, e.g. 'ABCDEFGHIJKLMNOP'. Or you could use 
% the char function. char(65:80) will return the letters A to P in a string 
letters = 'A':'P';

% We want the 12th letter, so we can index the string just like an array
letters(12) %Returns L

%% Find all the genes on chromosome 12

% One way to find all the chr12 rows: strncmp. This function compares the
% first n characters in a string, or in our case, a cell array of strings

YLGenes = strncmp('YL',S.textdata(3:end,1),2);

%Note that we consider rows 3:end to ignore the header row and the EWEIGHT
%row. This isn't really necessary, but it will help later on.

% The number of genes is just the number of ones in this logical array. We
% can count them with sum

nYLGenes = sum(YLGenes)

% This returns 484

% Another way to find chr12: Define a function handle and use cellfun.
% Function handles are objects that you can use to define mini functions
% The syntax is 
%
%   name = @(input) expression
%
% For example if you wanted a function that added two numbers together, you
% could write
%
%   myAdd = @(x,y) x + y;
%
% Then, you could call myAdd(3,4) just as if myAdd was a function you
% defined in a separate file.
%
% Here's how you could use a function handle to find the chr12 rows:
%
% 	hasL = @(s) s(2)=='L';
%   chr12Genes = cellfun(hasL,S.textdata(3:end,1));

% Note that we can't use the contains function, e.g.
%   
%   contains(S.textdata(:,1),'L'))
%
% since there are other rows that contain the letter 'L



%% Loop for aneuploid search

% To find the aneuploid chromosome, we need to find the average expression
% level for all the genes on each chromosome. We'll do this with a loop

nChrom = 16;
DNAlevel = zeros(nChrom,1);
for iChrom = 1:nChrom %(number of chromosomes
    
    % Find the right genes. Only consider rows 2:end to ignore the headers
    geneRows = strncmp(['Y' letters(iChrom)],S.textdata(2:end,1),2);
    
    % Some values in the data are labeled 'NaN' meaning 'Not a Number'
    % The normal mean function cannot handle NaN values, so we use nanmean.
    % This will simply ignore the NaN values when calculating a mean
    DNAlevel(iChrom) = nanmean(S.data(geneRows,DNAcolumn));
end

% To visualize the means, we use a simple plot. We also make sure that each
% data point has an XTick
figure
plot(DNAlevel,'o')
set(gca,'XTick',1:nChrom)
xlabel('Chromosome number')
ylabel('Mean DNA level')
% Clearly chr12 has an abnormal mean and is the aneuploid chromosome


%% Find RNA/DNA ratio

% Calculate the RNA to DNA ratio. First we take the mean across the 3 RNA
% columns and then use element-wise division
ratio = mean(S.data(2:end,RNAcolumns),2) ./ S.data(2:end,DNAcolumn);
YLrows = strncmp('YL',S.textdata(3:end,1),2);

% Remove NaN values
nanRows = ~isnan(ratio);
ratio = ratio(nanRows);
YLrows = find(YLrows(nanRows));

% Now plot using different color for chromosome12. We'l plot in three
% steps. The genes in chr1-11, then chr12, then chr13-16
x1 = 1:YLrows(1)-1;
x2 = YLrows;
x3 = YLrows(end)+1:length(ratio);

figure; 
hold on
plot(x1,ratio(x1),'b.-')
plot(x2,ratio(x2),'r.-')
plot(x3,ratio(x3),'b.-')
ylabel('RNA./DNA ratio')
xlabel('Index')
title('Raw data')
legend('Other chromosomes','chromosome 12')

% It's a bit hard to see anything with the outliers. We'll try smoothing 
% the data. We've chosen one of the "robust" methods that places lower 
% weights on outliers. You should try a few of the methods. See the 
% documentation of smooth for more options

window = 10; %Number of data points considered while smoothing
method = 'rlowess'; %Method
smoothRatio = smooth(ratio,window,method);

% Redo the plot
figure; 
hold on
plot(x1,smoothRatio(x1),'b.-')
plot(x2,smoothRatio(x2),'r.-')
plot(x3,smoothRatio(x3),'b.-')
ylabel('RNA./DNA ratio')
xlabel('Index')
title('Smoothed data')
legend('Other chromosomes','chromosome 12')

% What information can you see in this plot that you couldn't before? 
% What information can you no longer see?

