function randInts = myRandInt(lower,upper,nInts)
% MYRANDINT - Generator uniform random ints
%
%   R = myRandInt(X,Y,Z) generates Z random integers uniformly distributed
%   from integers X to Y
%

% There's a lot of error-checking that we could do to make sure the input
% is valid. For now we will just make sure the inputs are integers
if ~isnumeric([lower upper nInts]) || any(mod([lower upper nInts],1)~=0) 
    error('Inputs must be integers');
end

% Note that we used a double vertical line ||, which adds something called
% "short-circuiting." This means the second statement "any(mod(..." is only
% executed if the first statement is false. This is necessary here because
% if the inputs aren't numeric, then mod() will throw an error. This way,
% mod() is only called if we know that the values are numeric

% Also check to make sure nInts is positive
if nInts<1
    error('Size input must be a positive integer')
end

% Sort values in case the second value was lower than the first
range = sort([lower,upper],'ascend');

% Generate some uniformly random numbers from 0-1
randNums = rand(1,nInts);

% Multiply by the desired range plus 1. When combined with the ceil
% function, these numbers will now be integers from 1 to diff(range)+1
% The diff() function simply finds the difference between values, e.g.
% upper-lower
randInts = ceil(randNums * (diff(range)+1)); 

% Now we add on the lower value - 1 to move the values back into the
% correct range
randInts = randInts + range(1) - 1;